@@ -84,6 +84,8 @@ class automatumDataBokeh():
     # Rendering
     #
     #
+
+    
     def create_static_world_line_dict(self):
         """
             Creates the multi line data dict for rendering the static world. 
@@ -96,7 +98,7 @@ class automatumDataBokeh():
             "info": []          
         }
 
-        static_line_list = self.dataset.statWorld.get_lane_marking_dicts()
+        static_line_list = self.dataset.statWorld.get_lane_marking_dicts(with_centerline=False)
         for lane_markings in static_line_list:
             self.static_world_line_dict["xdata"].append(lane_markings["x_vec"])
             self.static_world_line_dict["ydata"].append(lane_markings["y_vec"])
@@ -129,21 +131,20 @@ class automatumDataBokeh():
             live_plot_dict["xdata"].append(x_vec)
             live_plot_dict["ydata"].append(y_vec)
             live_plot_dict["color"].append(self.uuid_color_mapping_dict[obj.UUID])
-            if(obj.tth_dict_vec is not None):
-                if(obj.tth_dict_vec[idx]["front_ego"] is None or obj.tth_dict_vec[idx]["front_ego"] < 0):
-                    tth = "-"
-                else:
+            tth = "-"
+            if(obj.tth_dict_vec is not None and len(obj.tth_dict_vec) > idx):
+                if(obj.tth_dict_vec[idx]["front_ego"] is not None and  obj.tth_dict_vec[idx]["front_ego"]>= 0):
                     tth = "%.3f" % obj.tth_dict_vec[idx]["front_ego"]
-            if(obj.ttc_dict_vec is not None):
-                if(obj.ttc_dict_vec[idx]["front_ego"] is None or obj.ttc_dict_vec[idx]["front_ego"] < 0):
-                    ttc = "-"
-                else:
+
+            ttc = "-"
+            if(obj.ttc_dict_vec is not None and len(obj.ttc_dict_vec) > idx): 
+                if(obj.ttc_dict_vec[idx]["front_ego"] is not None and obj.ttc_dict_vec[idx]["front_ego"] >= 0):
                     ttc = "%.3f" % obj.ttc_dict_vec[idx]["front_ego"]
 
 
-            live_plot_dict["info"].append("Type: %s -  UUID: %s - v_x: %.3f - v_y: %.3f - TTC Front: %s - TTH Front: %s" % (obj.type, obj.UUID, obj.vx_vec[idx], obj.vy_vec[idx], ttc, tth))
+            live_plot_dict["info"].append("UUID: %s - v_x: %.3f - v_y: %.3f - TTC Front: %s - TTH Front: %s" % (obj.UUID, obj.vx_vec[idx], obj.vy_vec[idx], ttc, tth))
 
-            if(0 in self.extra_plot_option_checkbox.active and obj.object_relation_dict_list is not None):
+            if(0 in self.extra_plot_option_checkbox.active and obj.object_relation_dict_list is not None and len(obj.object_relation_dict_list) > idx):
                 relation_dict = obj.object_relation_dict_list[idx]
                 for key, value in relation_dict.items():
                     if(value is not None and "front" in key):
@@ -159,7 +160,7 @@ class automatumDataBokeh():
                             live_plot_dict["color"].append("red")   
                         live_plot_dict["info"].append("%s from %s to %s" % (key, obj.UUID, value))
 
-            if(1 in self.extra_plot_option_checkbox.active and obj.distance_left_lane_marking is not None and obj.distance_right_lane_marking is not None):
+            if(1 in self.extra_plot_option_checkbox.active and obj.distance_left_lane_marking is not None and obj.distance_right_lane_marking is not None and len(obj.distance_left_lane_marking) > idx):
                 if(obj.distance_left_lane_marking[idx] >= 0):
                     x_vec, y_vec = generate_lines_to_lane_marking(obj.x_vec[idx], obj.y_vec[idx], obj.psi_vec[idx], obj.distance_left_lane_marking[idx], direction=1.0)                                                 
                     live_plot_dict["xdata"].append(x_vec)
