@@ -8,6 +8,12 @@ import os
 
 
 
+
+
+
+
+
+
 class xodrStaticWorld():
     """
         Representing the static world of dataset
@@ -40,7 +46,7 @@ class xodrStaticWorld():
 
 
 
-    def get_lane_marking_dicts(self, with_centerline=0):
+    def get_lane_marking_dicts(self, with_centerline=False):
         """
         This functions returns all lane markings of the static world. 
 
@@ -66,7 +72,7 @@ class xodrStaticWorld():
         """
         allLineList = list()
         for road in self.roads:
-            allLineList = allLineList + road.get_lane_marking_dicts()
+            allLineList = allLineList + road.get_lane_marking_dicts(with_centerline)
         
         return allLineList
 
@@ -84,6 +90,82 @@ class xodrStaticWorld():
 
         return None
 
+class xodrMiddleLine():
+    """This Class represents the middle lane between both sides of the road. 
+
+    :var x_vec: Vector of x position
+    :var y_vec: Vector of y position
+    :var offset: Lateral offset between the original reference line and the Middle line. 
+    
+    
+    """
+
+    def __init__(self, parent):
+
+        self.s_vec = list()
+        self.x_vec_center_line = list()
+        self.y_vec_center_line = list()
+        self.offset_vec = list()
+        self.geo_vec = list()
+
+        self.parent = parent 
+
+        # Actual Render Data
+        self.x_vec = list()
+        self.y_vec = list() 
+        self.off_vec = list()
+        self.theta_vec = list()
+
+    def add_discrete_reference_line_point(self, s, x, y, off, geo):
+        self.s_vec.append(s)
+        self.x_vec_center_line.append(x)
+        self.y_vec_center_line.append(y)
+        self.offset_vec.append(off)
+        self.geo_vec.append(geo)
+        
+
+    def sample_me(self):
+        self.s_vec = copy.copy(self.s_vec)
+
+
+        if(len(self.s_vec) != len(self.offset_vec) and len(self.s_vec) != len(self.offset_vec) and len(self.s_vec) != len(self.x_vec_center_line) and len(self.s_vec) != len(self.y_vec_center_line)):
+            raise Exception("Render of Lane, not all input values the same size")
+
+        for index, (s_sample, x_pos_ref_line, y_pos_ref_line, off_value, geo_ref_line) in enumerate(zip(self.s_vec, self.x_vec_center_line, self.y_vec_center_line, self.offset_vec, self.geo_vec)):
+            
+            if(index == 0 or last_geo != geo_ref_line):
+                segmentAngle = geo_ref_line.hdg + np.pi 
+            else:
+                dx = self.x_vec_center_line[index - 1] - x_pos_ref_line
+                dy = self.y_vec_center_line[index - 1] - y_pos_ref_line
+                if(abs(dx) < 0.0001 and abs(dy) < 0.0001):
+                    
+                    segmentAngle = geo_ref_line.hdg + np.pi 
+                    if(abs(segmentAngle - self.theta_vec[index - 1]) < 0.001):
+                        raise Exception("Inconsitcy in heading of roads")
+                else:
+                    segmentAngle = math.atan2(dy, dx)
+            last_geo = geo_ref_line
+
+
+            theta = (segmentAngle - np.pi / 2)
+
+            c, s = np.cos(theta), np.sin(theta)
+            R = np.array(((c, -s), (s, c)))
+
+            lineVec = np.matmul(R, np.array([1, 0])) * off_value
+            self.theta_vec.append(theta)
+            self.x_vec.append(x_pos_ref_line + lineVec[0])
+            self.y_vec.append(y_pos_ref_line + lineVec[1])
+
+
+    def get_lane_marking_dicts(self):
+        """ Returns the lane marking plotting vector for the center line"""
+        if(self.s_vec is None):
+            raise Exception("Please call sample_me before activating the plotting")
+
+        return [{"x_vec": list(self.x_vec), "y_vec": list(self.y_vec), "type": "centerline", "color": "white", "width": 0.12}]
+
 
 class xodrRoad():
     """
@@ -116,6 +198,7 @@ class xodrRoad():
         self.successor = None
 
         self.geometry = list()
+        self.middle_lines = list()
         self.lanesections = list()
         self.lane_offset = list()
         geometry_found = False
@@ -125,15 +208,21 @@ class xodrRoad():
                     if "geometry" in geoChild.tag:
                         geometry_found = True
                         self.geometry.append(xodrGeometry.create_geo_obj(geoChild))
+                        self.middle_lines.append(xodrMiddleLine(self))
                     else:
                         raise Exception("Unexpected XODR Format: Expected geometry, however given is %s" % geoChild.tag)
 
 
-
+            first_lane_section = True
             if "lanes" in child.tag:
-                for laneChild in child:
+                for idx, laneChild in enumerate(child):
+                    last_lane_section = idx == (len(child) - 1)
                     if "laneSection" in laneChild.tag:
-                        self.lanesections.append(xodrLaneSection(laneChild))
+                        try:
+                            self.lanesections.append(xodrLaneSection(laneChild, first_lane_section=first_lane_section, last_lane_section=last_lane_section))
+                            first_lane_section = False
+                        except Exception as e:
+                            raise Exception("While loading the lanes of road %d the following error occurs: %s" % (self.id, str(e)))
                     elif "laneOffset" in laneChild.tag:
                         self.lane_offset.append(xodrLaneOffset(laneChild))
 
@@ -159,7 +248,8 @@ class xodrRoad():
 
 
 
-    def get_lane_marking_dicts(self):
+    
+    def get_lane_marking_dicts(self, with_centerline=True):
         """
         This functions returns all lane markings corresponding to this static world object.
 
@@ -183,9 +273,12 @@ class xodrRoad():
         """
 
         lineList = list()
-
-        for geo in self.geometry:
-            lineList += geo.get_lane_marking_dicts()
+        if(with_centerline): 
+            for geo in self.geometry:
+                lineList += geo.get_lane_marking_dicts()
+        for middle_line in self.middle_lines:
+            lineList += middle_line.get_lane_marking_dicts() 
+        
         for laneSec in self.lanesections:
             lineList += laneSec.get_lane_marking_dicts()
 
@@ -213,8 +306,9 @@ class xodrRoad():
     def sample_me(self, sampleWidth):
         """ Calculates the sampled value (point vector) of this static world element. """
 
-        for geo in self.geometry:
+        for geo, middle_line in zip(self.geometry, self.middle_lines):
             geo.sample_me(sampleWidth)
+
             for s_value, x_value, y_value in zip(geo.s_vec, geo.x_vec, geo.y_vec):
 
                 for idx, lane_sec in enumerate(self.lanesections):
@@ -222,7 +316,10 @@ class xodrRoad():
                         next_lane_sec = self.lanesections[idx + 1]
                     else:
                         next_lane_sec = None
-                    lane_sec.check_if_s_value_is_relevant_and_add(s_value, self.get_lane_offset(s_value), x_value, y_value, geo, next_lane_sec)
+                    offset = self.get_lane_offset(s_value)
+                    lane_sec.check_if_s_value_is_relevant_and_add(s_value, offset, x_value, y_value, geo, next_lane_sec)
+                    middle_line.add_discrete_reference_line_point(s_value, x_value, y_value, offset, geo)
+            middle_line.sample_me()
         for lane_sec in self.lanesections:
             lane_sec.sample_me(geo)            
 
@@ -268,10 +365,14 @@ class xodrLaneSection():
 
         :var s: Length of this lane section element along the reference line
         :var lane: List of lane elements which belong to this lane section
+        :var first_lane_section: True if this lane section is the first one in the road
+        :var last_lane_section: True if this lane section is the last one in the road
     """
-    def __init__(self, xmlLaneSection):
+    def __init__(self, xmlLaneSection, first_lane_section=True, last_lane_section=True):
         """ Decodes the xmlTree of a LaneSection"""
         self.s = float(xmlLaneSection.attrib['s'])
+        self.first_lane_section = first_lane_section
+        self.last_lane_section = last_lane_section
         self.s_vec = list()
         self.off_vec = list()
         self.x_ref_line = list()
@@ -283,7 +384,13 @@ class xodrLaneSection():
             if("right" in child.tag or "left" in child.tag):
                 for leftrightLanes in child:
                     if "lane" in leftrightLanes.tag:
-                        self.lanes.append(xodrLane(leftrightLanes))
+                        new_lane = xodrLane(leftrightLanes)
+                        self.lanes.append(new_lane)
+                        if(not first_lane_section and new_lane.predecessor is not None and new_lane.id != new_lane.predecessor):
+                            raise Exception("A transition of the predecessor lane id (%d to %d) on the lane section (s: %f) within a road is not allowed" % (new_lane.id, new_lane.predecessor, self.s))
+                        if(not last_lane_section and new_lane.successor is not None and new_lane.id != new_lane.successor):
+                            raise Exception("A transition of the successor lane id (%d to %d) on the lane section (s: %f) within a road is not allowed" % (new_lane.id, new_lane.successor, self.s))
+
                     else:
                         raise Exception("Unexpected XODR Format: Expectation lane, given %s" % child.tag)
 
@@ -314,7 +421,8 @@ class xodrLaneSection():
 
         posLines.sort()
         l_normed = l - self.s
-        if(d > 0):
+        d_normed = d - lane_offset
+        if(d_normed > 0):
             curr_lane_offset = 0
             for idx, lane in enumerate(posLines):
                 if(idx == 0):
@@ -324,7 +432,7 @@ class xodrLaneSection():
                 if(curr_lane_offset >= d):
                     return lane.id
 
-        if(d < 0):
+        if(d_normed < 0):
             curr_lane_offset = 0
             for idx, lane in enumerate(negLines):
                 if(idx == 0):
@@ -492,8 +600,12 @@ class xodrLane():
             if("link" in laneChild.tag):
                 for link_child in laneChild:
                     if("predecessor" in link_child.tag):
+                        if(self.predecessor is not None):
+                            raise Exception("The lane %d has multiple predecessor, which is not allowed" % self.id)
                         self.predecessor = int(link_child.attrib['id'])
                     if("successor" in link_child.tag):
+                        if(self.successor is not None):
+                            raise Exception("The lane %d has multiple successor, which is not allowed" % self.id)                        
                         self.successor = int(link_child.attrib['id'])
         
             if("roadMark" in laneChild.tag and (laneChild.attrib['type'] == "broken" or laneChild.attrib['type'] == "solid")):
@@ -566,25 +678,9 @@ class xodrLane():
         for index, (s_sample, x_pos_ref_line, y_pos_ref_line, off_value, geo_ref_line) in enumerate(zip(s_vec, x_vec, y_vec, off_vec, geo_vec)):
             ds = s_sample - s_offset
             
-            lane_width = 0
-            lane_width_found = False
-            for width_index, width in enumerate(self.width_list):
-                if(width_index + 1 < len(self.width_list)):
-                    next_width_element_fits = ds < self.width_list[width_index + 1].s
-                else:
-                    # We have no next width element, so the current is applied for all given distance
-                    next_width_element_fits = True
-
-                if(ds >= width.s and next_width_element_fits):
-                    if(lane_width_found):
-                        raise Exception("We found mutliple lane width entry for one given s")
-                    lane_width_found = True
-                    lane_width = width.get_lane_width(ds)
-
-            if(not lane_width_found):
-                raise Exception("No lane width could be found")
 
 
+            lane_width = self.get_lane_width(ds, consider_sign=False)
             self.lane_width_vec.append(lane_width)
 
             
@@ -722,6 +818,7 @@ class xodrGeometry():
 
 
 
+
     def __lt__(self, other):
         """Overwrite left operator to enable a sorting based on start distance s
         """
@@ -893,16 +990,6 @@ class xodrGeometryPoly3(xodrGeometry):
         self.cV = float(xmlGeometry[0].attrib['cV'])
         self.dV = float(xmlGeometry[0].attrib['dV'])        
 
-    def get_track_and_orthogonal_distance(self, x, y):
-        """Calculates the distance between Reference Line alias Geometry and a given point
-        
-
-        :param x: Position x of given Point
-        :param y: Position y of given point
-        
-        """
-        raise NotImplementedError()
-
 
 
     def __str__(self):
